#!/bin/bash

#set -e  # Detiene el script cuando cualquier comando termina en error.
set -u  # Detiene el script ante cualquier intento de usar el valor de una variable no definida.

############################
# EUROCLIMA+  AEMET FIIAPP #
############################

# Este script genera las urls que permiten la creación on line de los scripts que bajarán los ficheros.
# Y comprueba el número de ficheros NetCDF que se bajarían bajando por separado los experimentos o bajándolos juntos. Y esto, para 
# cada nodo al que nos conectemos.
# Todo esto lo hace sin bajar realmetne los datos.


##############
# Usuario y contraseña para acceder a los nodos
##############
openid="https://esgf-node.ipsl.upmc.fr/esgf-idp/openid/nombre_usuario"
pass="mi_contraseña"

##############
# Carpeta de destino y fichero resumen
##############
dest_dir="/MASIVO/icx/ficherosCORDEX"   # Este directorio está destinado a albergar los ficheros de datos.
subdir_scripts="wget_scripts"   # Este es un subdirectorio de dest_dir. Es donde realmeten se guardarán los scripts wget.
resumen="$subdir_scripts/resumen_test_comercial.csv"

##############
# Listado de nodos
##############
sweden="esg-dn1.nsc.liu.se"
denmark="cordexesg.dmi.dk"      # No está operativo
germany="esgf-data.dkrz.de"
uk="esgf-index1.ceda.ac.uk"
france="esgf-node.ipsl.fr"      # No está operativo
spain="esgf-node.ipsl.upmc.fr"
india="esg-cccr.tropmet.res.in" # No está operativo

all_nodes="$sweden $denmark $germany $uk $france $spain $india"


##############
# Listado de opciones para generar el fichero wget
##############

# Para obtener un listado de todos los campos (facets) por los que podemos seleccionar los datos, se puede poner en el 
# navegador la siguiente url:
# https://esgf-node.jpl.nasa.gov/esg-search/search?project=CORDEX&facets=*&limit=0
# Y bajará un fichero XML que podemos explorar.

# Las opciones a usar son:
#   project=CORDEX
#   domain=CAM-44
#   experiment lo haremos iterar en sobre siguientes valores:
#       historical
#       rcp26
#       rcp45
#       rcp85
#   ensemble=r1i1p1
#   time_frequency=day
#   variable  la haremos iterar sobre los siguientes valores:
#       tasmax      (Daily Maximum Near-Surface Air Temperature)
#       tasmin      (Daily Minimum Near-Surface Air Temperature)
#       pr          (Precipitation)
#       mrro        (Total Runoff)
#       evspsbl     (Evaporation)
#       clt         (Total Cloud Fraction)
#       sfcWind     (Near-Surface Wind Speed)
#       wsgsmax     (Daily Maximum Near-Surface Wind Speed of Gust)
#       sfcWindmax  (Daily Maximum Near-Surface Wind Speed)
#       hurs        (Near-Surface Relative Humidity)

##############
# Parte fija de la url
##############
fixed_url="/esg-search/wget?project=CORDEX&domain=CAM-44&ensemble=r1i1p1&time_frequency=day"

##############
# Parte variable de la url
##############
experimentos="historical rcp26 rcp45 rcp85"
variables="tasmax tasmin pr mrro evspsbl clt sfcWind wsgsmax sfcWindmax hurs"

##############
# Bucle que genera los scripts wget y cuenta el nº de ficheros que bajaría (no hace una bajada real)
##############
cd "$dest_dir"
mkdir -p "$dest_dir/$subdir_scripts"
rm -f "$resumen"
echo "Variable;Nodo;Experimento;Núm ficheros" >> "$resumen"
# Cada script wget que nos bajemos, está limitado a bajar 1000 ficheros, número que se supera si tratamos de
# bajar todos los ficheros de todas las variables de una sola vez. Por ello es mejor bajar variable a variable.
# Esto, además de avisarlo el script wget que nos bajemos y que trate de superar los 1000 ficheros, se dice en:
# https://portal.enes.org/data/data-metadata-service/search-and-download/script-based-data-access
for variable in $variables
do
    for nodo in $all_nodes
    do
        echo
        # Primero descarga todos los experimentos de una sola vez
        experimentos_juntos="experiment="$(echo "${experimentos// /&experiment=}")
        url="https://$nodo/$fixed_url&$experimentos_juntos&variable=$variable"
        nombre_script_wget="$subdir_scripts/wget--cordex-americ_cent-$variable-todos_experimentos-$nodo.sh"
        wget --output-document="$nombre_script_wget" "$url"
        if [ "$?" -ne "0" ]; then
            num_fichs="0"
        else
            num_fichs=$(echo "$pass" | bash "$nombre_script_wget" -Hn -o "$openid" | grep ".nc " | wc -l)
        fi
        echo " Nº de ficheros para la variable $variable en el nodo $nodo: "$num_fichs
        echo "$variable;$nodo;todos;$num_fichs" >> "$resumen"
        # Ahora descarga cada experimento por separado
        for experimento in $experimentos
        do
            echo
            url="https://$nodo/$fixed_url&experiment=$experimento&variable=$variable"
            nombre_script_wget="$subdir_scripts/wget--cordex-americ_cent-$variable-$experimento-$nodo.sh"
            wget --output-document="$nombre_script_wget" "$url"
            if [ "$?" -ne "0" ]; then
                num_fichs="0"
            else
                num_fichs=$(echo "$pass" | bash "$nombre_script_wget" -Hn -o "$openid" | grep ".nc " | wc -l)
            fi
            echo " Nº de ficheros para la variable $variable y el experimento $experimento en el nodo $nodo: "$num_fichs
            echo "$variable;$nodo;$experimento;$num_fichs" >> "$resumen"
        done
    done

done


