#!/bin/bash

#set -e  # Detiene el script cuando cualquier comando termina en error.
set -u  # Detiene el script ante cualquier intento de usar el valor de una variable no definida.

#####################################
# Antonio Ángel Serrano de la Torre #
# FIIAPP   EUROCLIMA+  AEMET        #
#####################################

# Este script baja los datos CORDEX.
# Comprueba si cada script wget ya está creado y, si no lo está, lo baja.


##############
# Usuario y contraseña para acceder a los nodos
##############
openid="https://esgf-node.ipsl.upmc.fr/esgf-idp/openid/nombre_usuario"
pass="mi_contraseña"

##############
# Carpeta de destino y fichero resumen
##############
dest_dir="/ruta/donde/grabar/los/datos/que/se/descarguen"
subdir_scripts="wget_scripts"
resumen="$subdir_scripts/resumen_bajada.csv"

##############
# Listado de nodos
##############
sweden="esg-dn1.nsc.liu.se"
denmark="cordexesg.dmi.dk"      # No está operativo
germany="esgf-data.dkrz.de"
uk="esgf-index1.ceda.ac.uk"
france="esgf-node.ipsl.fr"      # No está operativo
spain="esgf-node.ipsl.upmc.fr"
india="esg-cccr.tropmet.res.in" # No está operativo


all_nodes="$sweden $denmark $germany $uk $france $spain $india"


##############
# Listado de opciones para generar el fichero wget
##############

# Para obtener un listado de todos los campos (facets) por los que podemos seleccionar los datos, se puede poner en el 
# navegador la siguiente url:
# https://esgf-node.jpl.nasa.gov/esg-search/search?project=CORDEX&facets=*&limit=0
# Y bajará un fichero XML que podemos explorar.

# En nuestro caso, las opciones a usar son:
#   project=CORDEX
#   domain=CAM-44
#   experiment lo haremos iterar en sobre siguientes valores:
#       historical
#       rcp26
#       rcp45
#       rcp85
#   ensemble=r1i1p1
#   time_frequency=day
#   variable  la haremos iterar sobre los siguientes valores:
#       tasmax      (Daily Maximum Near-Surface Air Temperature)
#       tasmin      (Daily Minimum Near-Surface Air Temperature)
#       pr          (Precipitation)
#       mrro        (Total Runoff)
#       evspsbl     (Evaporation)
#       clt         (Total Cloud Fraction)
#       sfcWind     (Near-Surface Wind Speed)
#       wsgsmax     (Daily Maximum Near-Surface Wind Speed of Gust)
#       sfcWindmax  (Daily Maximum Near-Surface Wind Speed)
#       hurs        (Near-Surface Relative Humidity)
#
# Los nombres de las variables que debemos usar, son los que aparecen en el menú
# "Variable" de la pantalla de selección de CORDEX

##############
# Parte fija de la url
##############
# Está formada por la cadena "/esg-search/" y por las selecciones fijas que se usan para todas las
# variables y todos los experimentos.
# Estas selecciones se hacen mediante "/wget?nombre_campo=valor_campo&nombre_campo2=valor_campo2 ...
fixed_url="/esg-search/wget?project=CORDEX&domain=CAM-44&ensemble=r1i1p1&time_frequency=day"

##############
# Parte variable de la url
##############
experimentos="historical rcp26 rcp45 rcp85"
variables="tasmax tasmin pr mrro evspsbl clt sfcWind wsgsmax sfcWindmax hurs"

##############
# Bucle que genera los scripts wget y baja los ficheros
##############
cd "$dest_dir"
mkdir -p "$dest_dir/$subdir_scripts"
rm -f "$resumen"
echo "Variable;Nodo;Experimento;Estatus" >> "$resumen"
# Cada script wget que nos bajemos, está limitado a bajar 1000 ficheros, número que se supera si tratamos de
# bajar todos los ficheros de todas las variables de una sola vez. Por ello es mejor bajar variable a variable.
# Esto, además de avisarlo el script wget que nos bajemos y que trate de superar los 1000 ficheros, se dice en:
# https://portal.enes.org/data/data-metadata-service/search-and-download/script-based-data-access
for variable in $variables
do
    for nodo in $spain
    do
        echo
        # Primero creamos la url que bajará el script wget que a su vez bajará los datos de esta variable
        experimentos_juntos="experiment="$(echo "${experimentos// /&experiment=}") # Descarga todos los experimentos de una sola vez
        url="https://$nodo/$fixed_url&$experimentos_juntos&variable=$variable"
        # Ahora establecemos el nombre que tendrá el script wget que vamos a bajar
        nombre_script_wget="$subdir_scripts/wget--cordex-americ_cent-$variable-todos_experimentos-$nodo.sh"
        # Ahora bajamos el script wget para esta variable
        if [ ! -f "$nombre_script_wget" ]; then
            wget --output-document="$nombre_script_wget" "$url"     # Esta es la orden exacta que lo baja
            if [ "$?" -ne "0" ]; then
                status="No descargado"
            else
                status="Descargado"
            fi
        else
            status="Ya existía (no se descarga otra vez)"
        fi
        # Escribimos un registro para esta variable, de cómo está funcionando todo
        echo "$variable;$nodo;todos;$status" >> "$resumen"
        # Ahora ejecutamos el script wget que hemos bajado. De este modo, se bajan los datos.
        set -e  # Detiene el script cuando cualquier comando termina en error.
        echo "$pass" | bash "$nombre_script_wget" -H -o "$openid"
        set +e  # Evita que se detenga el script cuando cualquier comando termina en error.
    done
done


